// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/driver/irq.c $
// $Rev: 43150 $
// $Date: 2018-07-02 17:33:22 -0500 (Mon, 02 Jul 2018) $

// 14HSAI4: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	u32	alt;
	u32	mask;
	u32	pstsr;

	pstsr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_pstsr_32);
	alt		= pstsr & (pstsr >> 16) & 0x3F3E;

	if (alt)
	{
		// Service the active interrupts.
		// Disable the LEVEL sensitive interrupts.
		mask	= (alt << 16) | (alt & 0x3E);
		os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_pstsr_32, 0, mask);

		// Resume any threads awiting any of these interrupts.
		gsc_wait_resume_irq_gsc(dev, alt);
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_pstsr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_pstsr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_pstsr_32, 0);

	gsc_irq_destroy(dev);
}


