// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/driver/main.h $
// $Rev: 53633 $
// $Date: 2023-08-29 09:41:27 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "14hsai4.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"14HSAI4"	// Upper case form of the below.
#define	DEV_NAME							"14hsai4"	// MUST AGREE WITH HSAI4_BASE_NAME

#define	DEV_VERSION							"2.8"		// FOR DEVICE SPECIFIC CODE ONLY!
// 2.8	Updated to support the 6.x series kernel.
//		Renamed all Auto_Calibrate content to Autocal for consistency.
//		Renamed all Auto_Cal_Sts content to Autocal_Status for consistency.
//		Updated the Initialize IOCTL service for consistency.
//		Updated the Autocalibration IOCTL service for consistency.
//		Updated the Input Buffer Clear IOCTL service for consistency.
//		Updated some comments for consistency.
//		Updated the make file for consistency.
// 2.7	Updated to Linux kernel 5.12.
//		Segregated PLX specific code.
//		Updated the autocalibration IOCTL service for consistency.
//		Removed unused macros.
//		Updated comments for consistency.
// 2.6	Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
// 2.5	Added support for the 5.x kernel series.
//		Removed compiler warnings for Fedora 31.
//		Updated initial file content for consistency.
//		Updated for porting to Windows.
// 2.4	Updated I/O code per changes to common code.
// 2.3	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
// 2.2	Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 2.1	Increased the size of the I/O transfer buffer.
// 2.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Fixed a bug in the HSAI4_IOCTL_AI_BUF_LEVEL service.
// 1.5	Reduced #include list in driver interface header.
// 1.4	Updated for the 3.x kernel.
//		Corrected a system timer count roll over bug in ioctl.c.
// 1.3	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
// 1.2	Eliminated the global dev_check_id() routine.
//		Removed compile warnings for Fedora 15.
// 1.1	Overhauled IOCTL based local interrupt implementation.
//		Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event/interrupt wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented explicit local interrupt support.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
//		Corrected HSAI4_IOCTL_GEN_INT_CLK_STATUS bug.
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					HSAI4_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					HSAI4_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						HSAI4_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMA, DMDMA
	s32					overflow_check;		// Check overflow when reading?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check underflow when reading?

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_sem_t			sem;				// Only one Tx or Rx at a time.
	os_mem_t			mem;				// I/O buffer.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;			// Analog Input read
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bctlr_32;	// 0x00 Board Control Register
		VADDR_T			gsc_pstsr_32;	// 0x04 Primary Status Register
		VADDR_T			gsc_idbr_32;	// 0x08 Input Data Buffer Register
		VADDR_T			gsc_ibcr_32;	// 0x0C Input Buffer Control Register
		VADDR_T			gsc_bufsr_32;	// 0x10 Buffer Size Register
		VADDR_T			gsc_rgar_32;	// 0x14 Rate Generator Adjust Register
		VADDR_T			gsc_rdr_32;		// 0x18 Rate Divisors Register
		VADDR_T			gsc_dfr_32;		// 0x1C Decimation Factor Register
		VADDR_T			gsc_cscr_32;	// 0x20 Clocking and Sync Control Register
		VADDR_T			gsc_bsizr_32;	// 0x24 Burst Size Register
		VADDR_T			gsc_psizr_32;	// 0x28 Pretrigger Size Register
		VADDR_T			gsc_bcfgr_32;	// 0x30 Board Configuration Register

	} vaddr;

	struct
	{
		s32				ai_chan_qty;		// The number of input channels on the device.
		s32				ai_chans_max;		// Maximum input channels supported.
		s32				autocal;			// Is autocalibrate supported?
		s32				autocal_ms;			// Maximum ms for autocalibrate

		s32				fgen_max;			// Maximum rate generator Fgen.
		s32				fgen_min;			// Minimum rate generator Fgen.
		u32				fifo_size_rx;		// Size of Rx FIFO - not the fill level.
		s32				fref_default;		// Default reference frequency.
		s32				fsamp_agg;			// The maximum aggrigate Fsamp limit.
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.
		s32				fsync_max;			// The maximum Fsync value.
		s32				fsync_min;			// The minimum Fsync value.

		u32				gsc_bcfgr_32;		// Board Configuration Register

		s32				initialize_ms;		// Maximum ms for initialize
		s32				input_config;		// High-Level, Wideband, Transformer
		s32				input_impedence;	// 50 ohm, 2M ohm

		s32				ndec_max;			// Maximum Ndec.
		s32				ndec_min;			// Minimum Ndec.
		s32				ndiv_clk_max;		// Maximum Ndiv-clk.
		s32				ndiv_clk_min;		// Minimum Ndiv-clk.
		s32				ndiv_sync_max;		// Maximum Ndiv-sync.
		s32				ndiv_sync_min;		// Minimum Ndiv-sync.
		s32				nref_max;			// Maximum Nref.
		s32				nref_min;			// Minimum Nref.
		s32				nvco_max;			// Maximum Nvco.
		s32				nvco_min;			// Minimum Nvco.

		s32				range_max;			// Maximum voltage range.
		s32				range_min;			// Minimum voltage range.
		s32				rate_gen_ms;		// Rate generator stttling time.

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
