// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/rxrate/perform.c $
// $Rev: 53622 $
// $Date: 2023-08-29 09:25:07 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_4M		(4 * 1024 * 1024)	// size of driver's transfer buffer



// variables ******************************************************************

static	u8	_data[_4M];



//*****************************************************************************
static int _configuration(const args_t* args)
{
	s32	chans;
	int	errs		= 0;
	s32	fsamp;
	s32	fsamp_agg;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= hsai4_query(args->fd, -1, 1, HSAI4_QUERY_CHANNEL_AI_QTY, &chans);
	errs	+= hsai4_query(args->fd, -1, 1, HSAI4_QUERY_FSAMP_AGG, &fsamp_agg);

	if (chans > 0)
		fsamp	= fsamp_agg / chans;
	else
		fsamp	= fsamp_agg;

	gsc_label("Configure");
	errs	+= hsai4_config_ai			(args->fd, -1, 0, fsamp);
	printf("%s  (", errs ? "FAIL <---" : "PASS");
	gsc_label_long_comma(fsamp);
	printf(" S/S)\n");

	errs	+= hsai4_rx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= hsai4_rx_io_overflow		(args->fd, -1, 1, HSAI4_IO_OVERFLOW_IGNORE, NULL);
	errs	+= hsai4_rx_io_underflow	(args->fd, -1, 1, HSAI4_IO_UNDERFLOW_IGNORE, NULL);
	errs	+= hsai4_fsamp_ai_report_all(args->fd, -1, 1, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= hsai4_read(args->fd, _data, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if (total < limit)
			continue;

		if (now.tv_sec < minimum.tv_sec)
			continue;

		if ((now.tv_sec == minimum.tv_sec) && (now.tv_usec < minimum.tv_usec))
			continue;

		us	= (now.tv_sec - begin.tv_sec) * 1000000L
			+ (now.tv_usec - begin.tv_usec);
		sps	= (total * 10000000L / 4 / us + 5) / 10;

		printf(	"PASS  (");
		gsc_label_long_comma(total / 4);
		printf(" Samples, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
		gsc_label_long_comma(sps);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_data);

	errs	+= _configuration(args);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= hsai4_ai_buf_clear		(args->fd, -1, 1);
	errs	+= hsai4_ai_buf_overflow	(args->fd, -1, 1, HSAI4_BUF_ERROR_CLEAR, NULL);
	errs	+= hsai4_ai_buf_underflow	(args->fd, -1, 1, HSAI4_BUF_ERROR_CLEAR, NULL);

	errs	+= _rate_test(args, get);

	errs	+= hsai4_ai_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= hsai4_ai_buf_underflow	(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();
	return(errs);
}



