// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/ai_buf_level.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Put some data in the FIFO: 16 samples

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_SYNC_SRC,
			/* arg		*/	HSAI4_AI_SYNC_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	8,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	s32			lvl;
	gsc_reg_t	reg;
	int			ret;

	errs	+= hsai4_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list);

	// We're only testing the level accuracy from 16 samples to zero.

	for (i = 16; (errs == 0) && (i >= 0); i--)
	{
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_LEVEL, &lvl);

		if (ret)
		{
			errs++;
			break;
		}
		else if (lvl != i)
		{
			errs++;
			printf("FAIL <---  (expected %d, got %ld)\n", i, (long) lvl);
			break;
		}

		reg.reg	= HSAI4_GSC_IDBR;
		ret		= hsai4_ioctl(fd, HSAI4_IOCTL_REG_READ, &reg);

		if (ret)
		{
			errs++;
			break;
		}
	}

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_buf_level_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_AI_BUF_LEVEL.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_level_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_AI_BUF_LEVEL");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


