// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/ai_buf_overflow.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		// Force a buffer overflow.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait while we get input.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we got an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},

		// Now clear the overflow and verify the results.

		{					// Manually clear the overflow.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},
		{					// Verify that there is no overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that there is an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},
		{				// Verify that IGNORE changes nothing.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_OVERFLOW,
			/* arg		*/	HSAI4_BUF_ERROR_IGNORE,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{				// Verify that clear REMOVES THE STATUS.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_OVERFLOW,
			/* arg		*/	HSAI4_BUF_ERROR_CLEAR,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	sts		= 0;

	errs	+= hsai4_initialize(fd, -1, 0);

	// Verify that the status is NO overflow.
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		sts	= -1;
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_OVERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != HSAI4_BUF_ERROR_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. overflow occurred but unexpected)\n",
					__LINE__);
		}
	}

	// Verify that the status IS an overflow.
	errs	+= service_ioctl_set_reg_list(fd, list_2);

	if (errs == 0)
	{
		sts	= -1;
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_OVERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != HSAI4_BUF_ERROR_YES)
		{
			errs++;
			printf(	"FAIL <---  (%d. overflow expected but did not occur)\n",
					__LINE__);
		}
	}

	// Verify that the status is NO overflow.
	errs	+= service_ioctl_set_reg_list(fd, list_3);

	if (errs == 0)
	{
		sts	= -1;
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_OVERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != HSAI4_BUF_ERROR_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. overflow occurred but unexpected)\n",
					__LINE__);
		}
	}

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_buf_overflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_AI_BUF_OVERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_overflow_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_AI_BUF_OVERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


