// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/ai_buf_thr_sts.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		// Put some data in the FIFO: 100 samples

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_SYNC_SRC,
			/* arg		*/	HSAI4_AI_SYNC_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	250,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Set and test the threshold level.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	HSAI4_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Set and test the threshold level.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	99,
			/* reg		*/	HSAI4_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{					// Set and test the threshold level.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	HSAI4_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	u32	sts;

	errs	+= hsai4_initialize(fd, -1, 0);

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSAI4_AI_BUF_THR_STS_CLEAR)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSAI4_AI_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	// Verify that the threshold status is SET.
	errs	+= service_ioctl_set_reg_list(fd, list_2);

	ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSAI4_AI_BUF_THR_STS_SET)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSAI4_AI_BUF_THR_STS_SET,
					(long) sts);
	}

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, list_3);

	ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSAI4_AI_BUF_THR_STS_CLEAR)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSAI4_AI_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_buf_thr_sts_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_AI_BUF_THR_STS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_thr_sts_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_AI_BUF_THR_STS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


