// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/ai_burst_mode.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_ENABLE,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x200,
			/* value	*/	0x200
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_DISABLE,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x200,
			/* value	*/	0x000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hsai4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Put some data in the FIFO: 16 samples, burst enabled

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_SYNC_SRC,
			/* arg		*/	HSAI4_AI_SYNC_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	8,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	16
		},


		// Put some data in the FIFO: burst disabled, buffer overflow

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hsai4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_burst_mode_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_AI_BURST_MODE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_burst_mode_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_AI_BURST_MODE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


