// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/ai_burst_status.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_SYNC_SRC,
			/* arg		*/	HSAI4_AI_SYNC_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_GEN_NDEC,
			/* arg		*/	1000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	1000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	sts		= 0;

	errs	+= hsai4_initialize(fd, -1, 0);

	// Verify that the status is IDLE.
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		sts	= -1;
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BURST_STATUS, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != HSAI4_BURST_STATUS_IDLE)
		{
			errs++;
			printf(	"FAIL <---  (%d. got %ld, should be %ld)\n",
					__LINE__,
					(long) sts,
					(long) HSAI4_BURST_STATUS_IDLE);
		}
	}

	// Verify that the status is ACTIVE.
	errs	+= service_ioctl_set_reg_list(fd, list_2);

	if (errs == 0)
	{
		sts	= -1;
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BURST_STATUS, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != HSAI4_BURST_STATUS_ACTIVE)
		{
			errs++;
			printf(	"FAIL <---  (%d. got %ld, should be %ld)\n",
					__LINE__,
					(long) sts,
					(long) HSAI4_BURST_STATUS_ACTIVE);
		}
	}

	// Verify that the status is IDLE.
	errs	+= service_ioctl_set_reg_list(fd, list_3);

	if (errs == 0)
	{
		sts	= -1;
		ret	= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BURST_STATUS, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != HSAI4_BURST_STATUS_IDLE)
		{
			errs++;
			printf(	"FAIL <---  (%d. got %ld, should be %ld)\n",
					__LINE__,
					(long) sts,
					(long) HSAI4_BURST_STATUS_IDLE);
		}
	}

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_burst_status_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_AI_BURST_STATUS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_burst_status_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_AI_BURST_STATUS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


