// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/ai_mode.c $
// $Rev: 53626 $
// $Date: 2023-08-29 09:28:45 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_MODE,
			/* arg		*/	HSAI4_AI_MODE_VREF,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x7,
			/* value	*/	0x3
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_MODE,
			/* arg		*/	HSAI4_AI_MODE_ZERO,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x7,
			/* value	*/	0x2
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_MODE,
			/* arg		*/	HSAI4_AI_MODE_SYSTEM,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x7,
			/* value	*/	0x0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	// The device user manual says this field has no effect on Transformer-Coupled device.

	errs	+= hsai4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	#define	VREF	0x3FF8
	#define	ZERO	0x2000

	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_MODE,
			/* arg		*/	HSAI4_BURST_MODE_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_SYNC_SRC,
			/* arg		*/	HSAI4_AI_SYNC_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_DATA_FORMAT,
			/* arg		*/	HSAI4_DATA_FORMAT_OFF_BIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	vref[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_MODE,
			/* arg		*/	HSAI4_AI_MODE_VREF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	1
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	zero[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_MODE,
			/* arg		*/	HSAI4_AI_MODE_ZERO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BURST_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	1
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	gsc_reg_t	parm;
	int			ret;
	s32			value;

	errs	+= hsai4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	// ZERO ===================================================================
	#define	ZERO_MAX	(ZERO + 0xFF)
	#define	ZERO_MIN	(ZERO - 0xFF)

	errs		+= service_ioctl_set_reg_list(fd, zero);
	parm.reg	= HSAI4_GSC_IDBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0x3FFF;

	if (ret)
	{
		errs++;
		printf(	"FAIL <---  (%d. hsai4_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < ZERO_MIN) || (value > ZERO_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. Zero error: 0x%03lX)\n",
				__LINE__,
				(long) value);
	}

	// VREF ===================================================================
	#define	VREF_MAX	(VREF + 0xFF)
	#define	VREF_MIN	(VREF - 0xFF)

	errs		+= service_ioctl_set_reg_list(fd, vref);
	parm.reg	= HSAI4_GSC_IDBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0x3FFF;

	if (ret)
	{
		errs++;
		printf(	"FAIL <---  (%d. hsai4_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < VREF_MIN) || (value > VREF_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. Vref error: 0x%03lX)\n",
				__LINE__,
				(long) value);
	}

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_mode_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_AI_MODE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_mode_test(int fd)
{
	s32	config	= HSAI4_INPUT_CONFIG_HIGH_LEVEL;
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_AI_MODE");
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_INPUT_CONFIG, &config);

	if (config == HSAI4_INPUT_CONFIG_TRANSFORMER)
	{
		printf("SKIPPED  (Testing limitation for -TR devices.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


