// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/data_format.c $
// $Rev: 53626 $
// $Date: 2023-08-29 09:28:45 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_DATA_FORMAT,
			/* arg		*/	HSAI4_DATA_FORMAT_OFF_BIN,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x40,
			/* value	*/	0x40
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_DATA_FORMAT,
			/* arg		*/	HSAI4_DATA_FORMAT_2S_COMP,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x40,
			/* value	*/	0x00
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hsai4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Configure the inputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_MODE,
			/* arg		*/	HSAI4_AI_MODE_VREF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Validate OFFSET BINARY.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_DATA_FORMAT,
			/* arg		*/	HSAI4_DATA_FORMAT_OFF_BIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_IDBR,
			/* mask		*/	0x3000,
			/* value	*/	0x3000
		},

		// Validate Two's Compliment.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_DATA_FORMAT,
			/* arg		*/	HSAI4_DATA_FORMAT_2S_COMP,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_IDBR,
			/* mask		*/	0x3000,
			/* value	*/	0x1000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hsai4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	data_format_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_DATA_FORMAT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int data_format_test(int fd)
{
	s32	config	= HSAI4_INPUT_CONFIG_HIGH_LEVEL;
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_DATA_FORMAT");
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_INPUT_CONFIG, &config);

	if (config == HSAI4_INPUT_CONFIG_TRANSFORMER)
	{
		printf("SKIPPED  (Testing limitation for -TR devices.)\n");
	}
	else
	{
		errs	+= _service_test(fd);
		errs	+= _function_test(fd);

		if (errs == 0)
			printf("PASS\n");
	}

	return(errs);
}


