// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/main.h $
// $Rev: 53626 $
// $Date: 2023-08-29 09:28:45 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <semaphore.h>
#include <time.h>

#include "14hsai4_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)						(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	index;			// device index
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of devices detected.
	s32	test_limit;		// for continuous testing

	int	fd;				// File descriptor for device to access.

	// Device Settings

} args_t;

typedef enum
{
	SERVICE_END_LIST,	// Ends list of service_data_t structures.
	SERVICE_NORMAL,
	SERVICE_REG_MOD,
	SERVICE_REG_READ,
	SERVICE_REG_SHOW,
	SERVICE_REG_TEST,
	SERVICE_SLEEP,
	SERVICE_SLEEP_MS,
	SERVICE_IOCTL_GET,
	SERVICE_IOCTL_SET
} service_t;

typedef struct
{
	service_t		service;
	int				cmd;	// IOCTL code
	s32				arg;	// The IOCTL data argument.
	u32				reg;	// The register to access. Use -1 to ignore.
	u32				mask;	// The register bits of interest.
	u32				value;	// The value expected for the bits of interest.
} service_data_t;



// prototypes *****************************************************************

int	ai_buf_clear_test(int fd);			// HSAI4_IOCTL_AI_BUF_CLEAR
int	ai_buf_enable_test(int fd);			// HSAI4_IOCTL_AI_BUF_ENABLE
int	ai_buf_level_test(int fd);			// HSAI4_IOCTL_AI_BUF_LEVEL
int	ai_buf_overflow_test(int fd);		// HSAI4_IOCTL_AI_BUF_OVERFLOW
int	ai_buf_thr_lvl_test(int fd);		// HSAI4_IOCTL_AI_BUF_THR_LVL
int	ai_buf_thr_sts_test(int fd);		// HSAI4_IOCTL_AI_BUF_THR_STS
int	ai_buf_underflow_test(int fd);		// HSAI4_IOCTL_AI_BUF_UNDERFLOW

int	ai_burst_marker_test(int fd);		// HSAI4_IOCTL_AI_BURST_MARKER
int	ai_burst_mode_test(int fd);			// HSAI4_IOCTL_AI_BURST_MODE
int	ai_burst_single_test(int fd);		// HSAI4_IOCTL_AI_BURST_SINGLE
int	ai_burst_size_test(int fd);			// HSAI4_IOCTL_AI_BURST_SIZE
int	ai_burst_status_test(int fd);		// HSAI4_IOCTL_AI_BURST_STATUS
int	ai_burst_trigger_test(int fd);		// HSAI4_IOCTL_AI_BURST_TRIGGER

int	ai_chan_sel_test(int fd);			// HSAI4_IOCTL_AI_CHAN_SEL
int	ai_chan_tag_test(int fd);			// HSAI4_IOCTL_AI_CHAN_TAG
int	ai_clock_src_test(int fd);			// HSAI4_IOCTL_AI_CLOCK_SRC
int	ai_clocking_test(int fd);			// HSAI4_IOCTL_AI_CLOCKING
int	ai_data_packing_test(int fd);		// HSAI4_IOCTL_AI_DATA_PACKING
int	ai_mode_test(int fd);				// HSAI4_IOCTL_AI_MODE
int	ai_pretrig_mode_test(int fd);		// HSAI4_IOCTL_AI_PRETRIG_MODE
int	ai_pretrig_size_test(int fd);		// HSAI4_IOCTL_AI_PRETRIG_SIZE
int	ai_pretrig_sts_test(int fd);		// HSAI4_IOCTL_AI_PRETRIG_STS
int	ai_range_test(int fd);				// HSAI4_IOCTL_AI_RANGE
int	ai_sol_chan_test(int fd);			// HSAI4_IOCTL_AI_SOL_CHAN
int	ai_sol_holdoff_test(int fd);		// HSAI4_IOCTL_AI_SOL_HOLDOFF
int	ai_sol_int_test(int fd);			// HSAI4_IOCTL_AI_SOL_INT
int	ai_sol_level_test(int fd);			// HSAI4_IOCTL_AI_SOL_LEVEL
int	ai_sync_src_test(int fd);			// HSAI4_IOCTL_AI_SYNC_SRC

int	autocal_test(int fd);				// HSAI4_IOCTL_AUTOCAL
int	autocal_status_test(int fd);		// HSAI4_IOCTL_AUTOCAL_STATUS

int	cbl_clk_sync_dir_test(int fd);		// HSAI4_IOCTL_CBL_CLK_SYNC_DIR

int	data_format_test(int fd);			// HSAI4_IOCTL_DATA_FORMAT

int	gen_int_clk_status_test(int fd);	// HSAI4_IOCTL_GEN_INT_CLK_STATUS
int	gen_ndec_test(int fd);				// HSAI4_IOCTL_GEN_NDEC
int	gen_ndiv_clk_test(int fd);			// HSAI4_IOCTL_GEN_NDIV_CLK
int	gen_ndiv_sync_test(int fd);			// HSAI4_IOCTL_GEN_NDIV_SYNC
int	gen_nref_test(int fd);				// HSAI4_IOCTL_GEN_NREF
int	gen_nvco_test(int fd);				// HSAI4_IOCTL_GEN_NVCO

int	initialize_test(int fd);			// HSAI4_IOCTL_INITIALIZE
int	irq_enable_test(int fd);			// HSAI4_IOCTL_IRQ_ENABLE

int	perform_tests(const args_t* args);

int	query_test(int fd);					// HSAI4_IOCTL_QUERY

int	reg_mod_test(int fd);				// HSAI4_IOCTL_REG_MOD
int	reg_read_test(int fd);				// HSAI4_IOCTL_REG_READ
int	reg_write_test(int fd);				// HSAI4_IOCTL_REG_WRITE
int	rx_io_abort_test(int fd);			// HSAI4_IOCTL_RX_IO_ABORT
int	rx_io_mode_test(int fd);			// HSAI4_IOCTL_RX_IO_MODE
int	rx_io_overflow_test(int fd);		// HSAI4_IOCTL_RX_IO_OVERFLOW
int	rx_io_timeout_test(int fd);			// HSAI4_IOCTL_RX_IO_TIMEOUT
int	rx_io_underflow_test(int fd);		// HSAI4_IOCTL_RX_IO_UNDERFLOW

int	service_ioctl_reg_get_list(int fd, const service_data_t* list);
int	service_ioctl_set_get_list(int fd, const service_data_t* list);
int	service_ioctl_set_reg_list(int fd, const service_data_t* list);

int	wait_cancel_test(int fd);			// HSAI4_IOCTL_WAIT_CANCEL
int	wait_event_test(int fd);			// HSAI4_IOCTL_WAIT_EVENT
int	wait_status_test(int fd);			// HSAI4_IOCTL_WAIT_STATUS

int	xcvr_test(int fd);					// HSAI4_IOCTL_XCVR



#endif
