// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/reg_mod.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg				mask
		{ HSAI4_GSC_BSIZR,	0x000FFFFF	},
		{ HSAI4_GSC_PSIZR,	0x0003FFF8	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;
	u32			value;

	for (i = 0; (errs == 0) && (i < SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is low.
			ret	= hsai4_ioctl(fd, HSAI4_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is high.
			ret	= hsai4_ioctl(fd, HSAI4_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= hsai4_ioctl(fd, HSAI4_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, hsai4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_REG_MOD");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


