// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/samples/sbtest/rx_io_underflow.c $
// $Rev: 51029 $
// $Date: 2022-04-27 11:39:59 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Sample Application: source file

#include "main.h"




//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		// Force an underflow.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_CLOCKING,
			/* arg		*/	HSAI4_AI_CLOCKING_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Induce the underflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_IDBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we got an underflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSAI4_GSC_BCTLR,
			/* mask		*/	0x10000,
			/* value	*/	0x10000
		},

		// Make I/O selections.

		{				// Ignore the underflow
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	HSAI4_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSAI4_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	HSAI4_IO_UNDERFLOW_CHECK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	data[1];
	int	errs	= 0;
	int	ret;

	errs	+= hsai4_initialize(fd, -1, 0);

	// Verify that IGNORE works.
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		ret	= hsai4_read(fd, data, 4);

		if (ret != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. hsai4_read(): returned %d, expected 0)\n",
					__LINE__,
					ret);
		}
	}

	// Verify that CHECK works.
	errs	+= service_ioctl_set_reg_list(fd, list_2);

	if (errs == 0)
	{
		ret	= hsai4_read(fd, data, 4);

		if (ret >= 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. hsai4_read(): returned %d, expected -1)\n",
					__LINE__,
					ret);
		}
	}

	errs	+= hsai4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_io_underflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSAI4_IOCTL_RX_IO_UNDERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rx_io_underflow_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSAI4_IOCTL_RX_IO_UNDERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


