// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_ai_buf_underflow.c $
// $Rev: 53627 $
// $Date: 2023-08-29 09:30:57 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsai4_ai_buf_underflow
*
*	Purpose:
*
*		Provide a visual wrapper for the HSAI4_IOCTL_AI_BUF_UNDERFLOW service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hsai4_ai_buf_underflow(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AI Buffer Underflow", index);

	ret		= hsai4_ioctl(fd, HSAI4_IOCTL_AI_BUF_UNDERFLOW, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case HSAI4_BUF_ERROR_NO:

			ptr	= "No";
			break;

		case HSAI4_BUF_ERROR_YES:

			ptr	= "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


