// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_ai_chan_sel.c $
// $Rev: 43155 $
// $Date: 2018-07-02 17:42:44 -0500 (Mon, 02 Jul 2018) $

// 14HSAI4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsai4_ai_chan_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the HSAI4_IOCTL_AI_CHAN_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hsai4_ai_chan_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	int	errs	= 0;
	s32	qty;
	int	ret;

	if (verbose)
		gsc_label_index("AI Channel Select", index);

	if (set != -1)
	{
		errs	= hsai4_query(fd, -1, 0, HSAI4_QUERY_CHANNEL_AI_QTY, &qty);
		set		= (qty == 4) ? (set & 0xF) : (set & 0x3);
	}

	ret		= hsai4_ioctl(fd, HSAI4_IOCTL_AI_CHAN_SEL, &set);
	errs	+= ret ? 1 : 0;

	if (verbose)
		printf("%s  (0x%lX)\n", errs ? "FAIL <---" : "PASS", (long) set);

	if (get)
		get[0]	= set;

	return(errs);
}


