// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_auto_cal_sts.c $
// $Rev: 53627 $
// $Date: 2023-08-29 09:30:57 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Utilities: source file

#include "main.h"



// Retained for backwards compatibility only:

/******************************************************************************
*
*	Function:	hsai4_auto_cal_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the HSAI4_IOCTL_AUTO_CAL_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hsai4_auto_cal_sts(int fd, int index, int verbose, s32* get)
{
	s32			acal	= 0;
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Autocalibrarion Status", index);

	errs	= hsai4_query(fd, -1, 0, HSAI4_QUERY_AUTO_CAL, &acal);

	if (acal)
	{
		ret		= hsai4_ioctl(fd, HSAI4_IOCTL_AUTO_CAL_STS, &set);
		errs	= ret ? 1 : 0;

		switch (set)
		{
			default:

				errs++;
				ptr	= buf;
				sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
				break;

			case HSAI4_AUTO_CAL_STS_ACTIVE:

				ptr	= "Active";
				break;

			case HSAI4_AUTO_CAL_STS_FAIL:

				ptr	= "Fail";
				break;

			case HSAI4_AUTO_CAL_STS_PASS:

				ptr	= "Pass";
				break;
		}

		if (verbose)
			printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);
	}
	else
	{
		if (verbose)
			printf("SKIPPED  (Autocalibration not supported on this device.)\n");

		set	= -1;
	}

	if (get)
		get[0]	= set;

	return(errs);
}


