// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_fsamp_ai.c $
// $Rev: 53627 $
// $Date: 2023-08-29 09:30:57 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



// data types *****************************************************************

typedef struct
{
	s32			fsamp_want;
	long double	fsamp_got_use;

	int			count;	// Number of full calculations.
	int			done;
	long double	delta_use;
	s32			fgen_max;
	s32			fgen_min;
	s32			fref;
	s32			fsamp_max;
	s32			fsamp_min;
	s32			ndec;
	s32			ndec_max;
	s32			ndec_min;
	s32			ndec_use;
	s32			ndiv_clk;
	s32			ndiv_clk_max;
	s32			ndiv_clk_min;
	s32			ndiv_clk_use;
	s32			nref;
	s32			nref_max;
	s32			nref_min;
	s32			nref_use;
	s32			nvco;
	s32			nvco_max;
	s32			nvco_min;
	s32			nvco_use;
} fsamp_t;



//*****************************************************************************
static void _data_eval(fsamp_t* data)
{
	long double	delta;
	long double	fsamp;

	for (;;)	// A convenience loop.
	{
		fsamp	= ((long double) data->fref * data->nvco)
				/ ((long double) data->nref * data->ndiv_clk * data->ndec);

		delta	= fsamp - data->fsamp_want;
		delta	= ABS(delta);

		if (data->count == 0)
			;
		else if (delta < data->delta_use)
			;
		else if ((delta == data->delta_use) && (data->nvco < data->nvco_use))
			;
		else
			break;

		data->count++;
		data->fsamp_got_use	= fsamp;
		data->delta_use		= delta;
		data->ndec_use		= data->ndec;
		data->ndiv_clk_use	= data->ndiv_clk;
		data->nref_use		= data->nref;
		data->nvco_use		= data->nvco;

		if (data->delta_use < 0.0001)
			data->done	= 1;

		break;
	}
}



//*****************************************************************************
static void _nvco_range(fsamp_t* data)
{
	s32	fgen;
	s32	ndec_max;
	s32	ndec_min;
	s32	nref_max;
	s32	nref_min;

	for (data->nvco = data->nvco_min; data->nvco <= data->nvco_max; data->nvco++)
	{
		nref_min	= (long long) data->fref * data->nvco / data->fgen_max - 1;
		nref_max	= (long long) data->fref * data->nvco / data->fgen_min + 1;
		nref_min	= LIMIT_RANGE(nref_min, data->nref_min, data->nref_max);
		nref_max	= LIMIT_RANGE(nref_max, data->nref_min, data->nref_max);

		for (data->nref = nref_min; data->nref <= nref_max; data->nref++)
		{
			fgen	= (s32) ((long long) data->fref * data->nvco / data->nref);

			if (data->count == 0)
				;
			else if ((fgen < data->fgen_min) || (fgen > data->fgen_max))
				continue;

			for (data->ndiv_clk = data->ndiv_clk_max; data->ndiv_clk >= data->ndiv_clk_min; data->ndiv_clk--)
			{
				ndec_min	= (s32) (((long double) data->fref * data->nvco)
							/ ((long long) data->nref * data->ndiv_clk_max * data->fsamp_want))
							- 1;
				ndec_max	= (s32) (((long double) data->fref * data->nvco)
							/ ((long long) data->nref * data->ndiv_clk_min * data->fsamp_want))
							+ 1;
				ndec_min	= LIMIT_RANGE(ndec_min, data->ndec_min, data->ndec_max);
				ndec_max	= LIMIT_RANGE(ndec_max, data->ndec_min, data->ndec_max);

				for (data->ndec = ndec_min; data->ndec <= ndec_max; data->ndec++)
				{
					_data_eval(data);

					if (data->done)
						break;
				}

				if (data->done)
					break;
			}

			if (data->done)
				break;
		}

		if (data->done)
			break;
	}
}



//*****************************************************************************
int hsai4_fsamp_ai_compute(
	int		fd,
	int		index,
	int		verbose,
	s32		fsamp,
	s32*	nvco,
	s32*	nref,
	s32*	ndiv_clk,
	s32*	ndec,
	double*	rate)
{
	fsamp_t	data;
	int		errs	= 0;

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	memset(&data, 0, sizeof(data));
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_FREF_DEFAULT, &data.fref);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_FGEN_MAX, &data.fgen_max);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_FGEN_MIN, &data.fgen_min);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_FSAMP_MAX, &data.fsamp_max);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_FSAMP_MIN, &data.fsamp_min);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NDEC_MAX, &data.ndec_max);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NDEC_MIN, &data.ndec_min);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NDIV_CLK_MAX, &data.ndiv_clk_max);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NDIV_CLK_MIN, &data.ndiv_clk_min);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NREF_MAX, &data.nref_max);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NREF_MIN, &data.nref_min);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NVCO_MAX, &data.nvco_max);
	errs	+= hsai4_query(fd, -1, 0, HSAI4_QUERY_NVCO_MIN, &data.nvco_min);

	data.delta_use	= FLT_MAX;
	data.fsamp_want	= LIMIT_RANGE(fsamp, data.fsamp_min, data.fsamp_max);

	if (errs == 0)
		_nvco_range(&data);

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma((long) data.fref);
		printf(", Nvco %ld", (long) data.nvco_use);
		printf(", Nref %ld", (long) data.nref_use);
		printf(", Ndiv-clk %ld", (long) data.ndiv_clk_use);
		printf(", Ndec %ld", (long) data.ndec_use);
		printf(", Fsamp %.3lf", (double) data.fsamp_got_use);
		printf(")\n");
		fflush(stdout);
	}

	if (nvco)
		nvco[0]		= data.nvco_use;

	if (nref)
		nref[0]		= data.nref_use;

	if (ndiv_clk)
		ndiv_clk[0]	= data.ndiv_clk_use;

	if (ndec)
		ndec[0]		= data.ndec_use;

	if (rate)
		rate[0]	= data.fsamp_got_use;

	return(errs);
}



/******************************************************************************
*
*	Function:	hsai4_fsamp_ai_get
*
*	Purpose:
*
*		Determine the input sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		title	Show a title?
*
*		fsamp	Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsai4_fsamp_ai_get(int fd, int index, int verbose, int title, s32* fsamp)
{
	s32		enable;
	int		errs	= 0;
	float	fgen;
	s32		fref;
	s32		ndec;
	s32		ndiv;
	s32		nref;
	s32		nvco;
	s32		set;
	s32		src;

	if ((verbose) && (title))
		gsc_label("AI Sample Rate");

	errs	+= hsai4_ai_clock_src	(fd, -1, 0, -1, &src);
	errs	+= hsai4_ai_clocking	(fd, -1, 0, -1, &enable);
	errs	+= hsai4_gen_ndec		(fd, -1, 0, -1, &ndec);
	errs	+= hsai4_gen_ndiv_clk	(fd, -1, 0, -1, &ndiv);
	errs	+= hsai4_gen_nref		(fd, -1, 0, -1, &nref);
	errs	+= hsai4_gen_nvco		(fd, -1, 0, -1, &nvco);
	errs	+= hsai4_query			(fd, -1, 0, HSAI4_QUERY_FREF_DEFAULT, &fref);

	if (errs)
	{
		set	= 0;

		if (verbose)
			printf("FAIL <---  (%d. service errors)\n", __LINE__);
	}
	else if (enable != HSAI4_AI_CLOCKING_ENABLE)
	{
		set	= 0;

		if (verbose)
			printf("0 S/S  (Clocking is disabled.)\n");
	}
	else if (src != HSAI4_AI_CLOCK_SRC_INTERNAL)
	{
		set	= 0;

		if (verbose)
			printf("0 S/S  (Source is external clock.)\n");
	}
	else
	{
		fgen	= ((float) fref * nvco) / nref;
		set		= (s32) (fgen / ndiv / ndec);

		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (Source = Internal, Fref = ");
			gsc_label_long_comma(fref);
			printf(" Hz, Nvco = ");
			gsc_label_long_comma(nvco);
			printf(", Nref = ");
			gsc_label_long_comma(nref);
			printf(", Ndiv-clk = ");
			gsc_label_long_comma(ndiv);
			printf(", ndec = ");
			gsc_label_long_comma(ndec);
			printf(")\n");
		}
	}

	if (fsamp)
		fsamp[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	hsai4_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the input sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The index of the channel of interest.
*
*		fsamp	Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsai4_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* fsamp)
{
	s32		bit			= 0x1L << chan;
	char	buf[64];
	int		errs;
	s32		select;
	s32		set			= 0;

	if (verbose)
	{
		sprintf(buf, "AI Channel %d", chan);
		gsc_label_index(buf, index);
	}

	errs	= hsai4_ai_chan_sel(fd, -1, 0, -1, &select);

	if (select & bit)
	{
		// The channel is enabled.
		errs	+= hsai4_fsamp_ai_get(fd, index, verbose, 0, &set);
	}
	else
	{
		// The channel is disabled.
		set	= 0;

		if (verbose)
			printf("0 S/S  (Channel Disabled)\n");
	}

	if (fsamp)
		fsamp[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	hsai4_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the input sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Report the overall sample rate here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsai4_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chans;
	int	chan;
	int	errs;
	s32	sps		= 0;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("AI Sample Rates", index);

	errs	= hsai4_query(fd, -1, 0, HSAI4_QUERY_CHANNEL_AI_QTY, &chans);

	if (errs)
	{
		if (verbose)
			printf("\n");
	}
	else
	{
		if (verbose)
		{
			printf("\n");
			gsc_label_level_inc();
		}

		for (chan = 0; chan < chans; chan++)
		{
			sps		= 0;
			errs	+= hsai4_fsamp_ai_report(fd, -1, verbose, chan, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall AI Sample Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
			gsc_label_level_dec();
		}
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


