// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_fsync_ai.c $
// $Rev: 51031 $
// $Date: 2022-04-27 11:42:21 -0500 (Wed, 27 Apr 2022) $

// 14HSAI4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsai4_fsync_ai_get
*
*	Purpose:
*
*		Determine the input SYNC rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsai4_fsync_ai_get(int fd, int index, int verbose, s32* get)
{
	s32		clk_src;
	s32		enable;
	int		errs	= 0;
	float	fgen;
	s32		fref;
	s32		ndiv_sync;
	s32		nref;
	s32		nvco;
	s32		set;
	s32		sync_src;

	if (verbose)
		gsc_label("AI Sync Rate");

	errs	+= hsai4_ai_clock_src	(fd, -1, 0, -1, &clk_src);
	errs	+= hsai4_ai_clocking	(fd, -1, 0, -1, &enable);
	errs	+= hsai4_gen_nref		(fd, -1, 0, -1, &nref);
	errs	+= hsai4_gen_nvco		(fd, -1, 0, -1, &nvco);
	errs	+= hsai4_ai_sync_src	(fd, -1, 0, -1, &sync_src);
	errs	+= hsai4_gen_ndiv_sync	(fd, -1, 0, -1, &ndiv_sync);
	errs	+= hsai4_query			(fd, -1, 0, HSAI4_QUERY_FREF_DEFAULT, &fref);

	if (errs)
	{
		set	= 0;

		if (verbose)
			printf("FAIL <---  (%d. service errors)\n", __LINE__);
	}
	else if (enable != HSAI4_AI_CLOCKING_ENABLE)
	{
		set	= 0;

		if (verbose)
			printf("0 S/S  (Clocking is disabled.)\n");
	}
	else if (sync_src != HSAI4_AI_SYNC_SRC_INTERNAL)
	{
		set	= 0;

		if (verbose)
			printf("0 S/S  (Source is external sync.)\n");
	}
	else if (clk_src != HSAI4_AI_CLOCK_SRC_INTERNAL)
	{
		set	= 0;

		if (verbose)
			printf("0 S/S  (Source is external clock.)\n");
	}
	else if (verbose)
	{
		fgen	= ((float) fref * nvco) / nref;
		set		= (s32) (fgen / ndiv_sync);
		gsc_label_long_comma(set);
		printf(" S/S  (Sources = Internal, Fref = ");
		gsc_label_long_comma(fref);
		printf(" Hz, Nvco = ");
		gsc_label_long_comma(nvco);
		printf(", Nref = ");
		gsc_label_long_comma(nref);
		printf(", Ndiv-sync = ");
		gsc_label_long_comma(ndiv_sync);
		printf(")\n");
	}
	else
	{
		set	= 0;
	}

	if (get)
		get[0]	= set;

	return(errs);
}


