// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_query.c $
// $Rev: 53627 $
// $Date: 2023-08-29 09:30:57 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsai4_query
*
*	Purpose:
*
*		Provide a visual wrapper for the HSAI4_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hsai4_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case HSAI4_QUERY_AI_RANGE_MAX:		ptr	= "Max Voltaget Range";		break;
		case HSAI4_QUERY_AI_RANGE_MIN:		ptr	= "Min Voltaget Range";		break;
		case HSAI4_QUERY_AUTOCAL:			ptr	= "Autocal Supported";		break;
		case HSAI4_QUERY_AUTOCAL_MS:		ptr	= "Autocal Duration";		break;
		case HSAI4_QUERY_CHANNEL_AI_MAX:	ptr	= "Max Channels";			break;
		case HSAI4_QUERY_CHANNEL_AI_QTY:	ptr	= "Installed Channels";		break;
		case HSAI4_QUERY_COUNT:				ptr	= "Query Options";			break;
		case HSAI4_QUERY_DEVICE_TYPE:		ptr	= "Device Type";			break;
		case HSAI4_QUERY_FGEN_MAX:			ptr	= "Fgen Min";				break;
		case HSAI4_QUERY_FGEN_MIN:			ptr	= "Fgen Max";				break;
		case HSAI4_QUERY_FIFO_SIZE_RX:		ptr	= "Rx FIFO Size";			break;
		case HSAI4_QUERY_FREF_DEFAULT:		ptr	= "Fref Default";			break;
		case HSAI4_QUERY_FSAMP_AGG:			ptr	= "Fsamp Aggrigate Max";	break;
		case HSAI4_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";				break;
		case HSAI4_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";				break;
		case HSAI4_QUERY_FSYNC_MAX:			ptr	= "Fsync Max";				break;
		case HSAI4_QUERY_FSYNC_MIN:			ptr	= "Fsync Min";				break;
		case HSAI4_QUERY_INIT_MS:			ptr	= "Init. Duration";			break;
		case HSAI4_QUERY_INPUT_CONFIG:		ptr	= "Input Config";			break;
		case HSAI4_QUERY_INPUT_IMPED:		ptr	= "Input Impedence";		break;
		case HSAI4_QUERY_NDEC_MAX:			ptr	= "Ndec Min";				break;
		case HSAI4_QUERY_NDEC_MIN:			ptr	= "Ndec Min";				break;
		case HSAI4_QUERY_NDIV_CLK_MAX:		ptr	= "Clock: Ndiv Max";		break;
		case HSAI4_QUERY_NDIV_CLK_MIN:		ptr	= "Clock: Ndiv Min";		break;
		case HSAI4_QUERY_NDIV_SYNC_MAX:		ptr	= "Sync: Ndiv Max";			break;
		case HSAI4_QUERY_NDIV_SYNC_MIN:		ptr	= "Sync: Ndiv Min";			break;
		case HSAI4_QUERY_NREF_MAX:			ptr	= "Nref Max";				break;
		case HSAI4_QUERY_NREF_MIN:			ptr	= "Nref Min";				break;
		case HSAI4_QUERY_NVCO_MAX:			ptr	= "Nvco Max";				break;
		case HSAI4_QUERY_NVCO_MIN:			ptr	= "Nvco Min";				break;
		case HSAI4_QUERY_RATE_GEN_MS:		ptr	= "Rate Gen Settling";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= hsai4_ioctl(fd, HSAI4_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case HSAI4_QUERY_CHANNEL_AI_MAX:
		case HSAI4_QUERY_CHANNEL_AI_QTY:
		case HSAI4_QUERY_COUNT:
		case HSAI4_QUERY_NDEC_MAX:
		case HSAI4_QUERY_NDEC_MIN:
		case HSAI4_QUERY_NDIV_CLK_MAX:
		case HSAI4_QUERY_NDIV_CLK_MIN:
		case HSAI4_QUERY_NDIV_SYNC_MAX:
		case HSAI4_QUERY_NDIV_SYNC_MIN:
		case HSAI4_QUERY_NVCO_MAX:
		case HSAI4_QUERY_NVCO_MIN:
		case HSAI4_QUERY_NREF_MAX:
		case HSAI4_QUERY_NREF_MIN:
		case HSAI4_QUERY_FSYNC_MAX:
		case HSAI4_QUERY_FSYNC_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case HSAI4_QUERY_FGEN_MAX:
		case HSAI4_QUERY_FGEN_MIN:
		case HSAI4_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case HSAI4_QUERY_AUTOCAL_MS:
		case HSAI4_QUERY_INIT_MS:
		case HSAI4_QUERY_RATE_GEN_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case HSAI4_QUERY_FSAMP_AGG:
		case HSAI4_QUERY_FSAMP_MAX:
		case HSAI4_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case HSAI4_QUERY_AUTOCAL:

			strcpy(buf, set ? "Supported" : "Not Supported");
			break;

		case HSAI4_QUERY_FIFO_SIZE_RX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case HSAI4_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_14HSAI4)
			{
				strcpy(buf, "14HSAI4");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case HSAI4_QUERY_AI_RANGE_MAX:
		case HSAI4_QUERY_AI_RANGE_MIN:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HSAI4_AI_RANGE_0_625V:	strcpy(buf, "+- 0.625 Volts");	break;
				case HSAI4_AI_RANGE_1_25V:	strcpy(buf, "+- 1.25 Volts");	break;
				case HSAI4_AI_RANGE_2_5V:	strcpy(buf, "+- 2.5 Volts");	break;
				case HSAI4_AI_RANGE_5V:		strcpy(buf, "+- 5 Volts");		break;
				case HSAI4_AI_RANGE_10V:	strcpy(buf, "+- 10 Volts");		break;
				case HSAI4_AI_RANGE_1_0VPP:	strcpy(buf, "1.0 Vp-p");		break;
				case HSAI4_AI_RANGE_2_0VPP:	strcpy(buf, "2.0 Vp-p");		break;
			}

			break;

		case HSAI4_QUERY_INPUT_CONFIG:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HSAI4_INPUT_CONFIG_HIGH_LEVEL:		strcpy(buf, "High Level");	break;
				case HSAI4_INPUT_CONFIG_WIDEBAND:		strcpy(buf, "Wideband");	break;
				case HSAI4_INPUT_CONFIG_TRANSFORMER:	strcpy(buf, "Transformer");	break;
			}

			break;

		case HSAI4_QUERY_INPUT_IMPED:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HSAI4_INPUT_IMPED_2M:	strcpy(buf, "2M Ohm");	break;
				case HSAI4_INPUT_IMPED_50:	strcpy(buf, "50 Ohm");	break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


