// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_reg.c $
// $Rev: 53627 $
// $Date: 2023-08-29 09:30:57 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Utilities: source file

#include "main.h"

// For code validation script.
// if (verbose)
// if (verbose)



// macros *********************************************************************

#define	_GSC_REG(a)		"GSC " #a, HSAI4_GSC_##a, 0, 0
#define	WIDTH			33



//*****************************************************************************
static int _bctlr_detail(int fd, int supported, u32 value, int width)
{
	#define	BCFGR_IC_MASK		0x60000
	#define	BCFGR_IC_HIGH_LEVEL	0x00000
	#define	BCFGR_IC_WIDEBAND	0x20000

	static const char*	range_hl[]	=
	{
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts",
		"RESERVED"
	};

	static const char*	range_wb[]	=
	{
		"+-0.625 Volts",
		"+-1.25 Volts",
		"+-2.5 Volts",
		"RESERVED"
	};

	static const char*	range_xformar[]	=
	{
		"1.0 Vp-p",
		"2.0 Vp-p",
		"2.0 Vp-p",
		"RESERVED"
	};

	static const char*	aim[]	=
	{
		"Analog Inputs",
		"RESERVED",
		"Zero Test",
		"Vref Test",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	disabled[]	= { "Disabled",			"Enabled"		};
	static const char*	fail[]		= { "Fail",				"Pass"			};
	static const char*	idle[]		= { "Idle",				"Active"		};
	static const char*	no[]		= { "No",				"Yes"			};
	static const char*	ttl[]		= { "TTL",				"LVDS"			};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"	};

	u32				bcfgr;
	int				errs;
	const char**	range;

	width	+= 10;
	errs	= hsai4_reg_read(fd, -1, 0, HSAI4_GSC_BCFGR, &bcfgr);

	if ((bcfgr & BCFGR_IC_MASK) == BCFGR_IC_HIGH_LEVEL)
		range	= range_hl;
	else if ((bcfgr & BCFGR_IC_MASK) == BCFGR_IC_WIDEBAND)
		range	= range_wb;
	else
		range	= range_xformar;

	gsc_reg_field_show(width, WIDTH, value, 31, 19, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, disabled,	"Data Packing"		);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, no,			"Buffer Overflow"	);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, no,			"Buffer Underflow"	);
	gsc_reg_field_show(width, WIDTH, value, 15, 15, 1, idle,		"Initialize"		);
	gsc_reg_field_show(width, WIDTH, value, 14, 14, 1, fail,		"Autocal Status"	);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, idle,		"Autocal"			);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, idle,		"Input Sync"		);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, no,			"Single Burst"		);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, disabled,	"Burst Marker"		);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, disabled,	"Bursting"			);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, disabled,	"ADC Clocking"		);
	gsc_reg_field_show(width, WIDTH, value,  7,  7, 1, ttl,			"Transceivers"		);
	gsc_reg_field_show(width, WIDTH, value,  6,  6, 1, twos,		"Data Format"		);
	gsc_reg_field_show(width, WIDTH, value,  5,  4, 1, range,		"Range"				);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value,  2,  0, 1, aim,			"Input Mode"		);

	return(errs);
}



//*****************************************************************************
static int _pstsr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]	= { "Clear",	"Set"		};
	static const char*	idle[]	= { "Idle",		"Active"	};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 30, 1, NULL,	"Reserved"							);
	gsc_reg_field_show(width, WIDTH, value, 29, 29, 1, idle,	"Pretrigger Armed"					);
	gsc_reg_field_show(width, WIDTH, value, 28, 28, 1, idle,	"Buffer Threshold Flag Hi->Low"		);
	gsc_reg_field_show(width, WIDTH, value, 27, 27, 1, idle,	"Buffer Threshold Flag Low->Hi"		);
	gsc_reg_field_show(width, WIDTH, value, 26, 26, 1, idle,	"Input Burst Completed"				);
	gsc_reg_field_show(width, WIDTH, value, 25, 25, 1, idle,	"Input Burst Triggered"				);
	gsc_reg_field_show(width, WIDTH, value, 24, 24, 1, idle,	"Autocal Done"						);
	gsc_reg_field_show(width, WIDTH, value, 23, 22, 1, NULL,	"Reserved"							);
	gsc_reg_field_show(width, WIDTH, value, 21, 21, 1, idle,	"Buffer Over/Underflow"				);
	gsc_reg_field_show(width, WIDTH, value, 20, 20, 1, idle,	"Burst Pretrigger-Armed Interval"	);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, idle,	"Buffer Threshold Flag"				);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, idle,	"Triggered Burst In Progress"		);
	gsc_reg_field_show(width, WIDTH, value, 17, 17, 1, idle,	"Autocal In Progress"				);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, NULL,	"Reserved"							);

	gsc_reg_field_show(width, WIDTH, value, 15, 14, 1, NULL,	"Reserved"							);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, clear,	"Pretrigger Armed"					);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, clear,	"Buffer Threshold Flag Hi->Low"		);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, clear,	"Buffer Threshold Flag Low->Hi"		);
	gsc_reg_field_show(width, WIDTH, value, 10, 10, 1, clear,	"Input Burst Completed"				);
	gsc_reg_field_show(width, WIDTH, value,  9,  9, 1, clear,	"Input Burst Triggered"				);
	gsc_reg_field_show(width, WIDTH, value,  8,  8, 1, clear,	"Autocal Done"						);
	gsc_reg_field_show(width, WIDTH, value,  7,  6, 1, NULL,	"Reserved"							);
	gsc_reg_field_show(width, WIDTH, value,  5,  5, 1, clear,	"Buffer Over/Underflow"				);
	gsc_reg_field_show(width, WIDTH, value,  4,  4, 1, clear,	"Burst Pretrigger-Armed Interval"	);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, clear,	"Buffer Threshold Flag"				);
	gsc_reg_field_show(width, WIDTH, value,  2,  2, 1, clear,	"Triggered Burst In Progress"		);
	gsc_reg_field_show(width, WIDTH, value,  1,  1, 1, clear,	"Autocal In Progress"				);
	gsc_reg_field_show(width, WIDTH, value,  0,  0, 1, NULL,	"Reserved"							);

	return(0);
}



//*****************************************************************************
static int _idbr_detail(int fd, int supported, u32 value, int width)
{
	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 16, 1, NULL,	"Data"	);
	gsc_reg_field_show(width, WIDTH, value, 15,  0, 1, NULL,	"Data"	);

	return(0);
}



//*****************************************************************************
static int _ibcr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	enabled[]	= { "Enabled",	"Disabled"	};
	static const char*	idle[]		= { "Idle",		"Active"	};
	static const char*	no[]		= { "No",		"Yes"		};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 22, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 21, 21, 1, enabled,	"Channel Tag"		);
	gsc_reg_field_show(width, WIDTH, value, 20, 20, 1, no,		"Buffer Disable"	);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, idle,	"Threshold Flag"	);
	gsc_reg_field_show(width, WIDTH, value, 18, 18, 1, idle,	"Clear Buffer"		);
	gsc_reg_field_show(width, WIDTH, value, 17,  0, 1, NULL,	"Threshold Level"	);

	return(0);
}



//*****************************************************************************
static int _bufsr_detail(int fd, int supported, u32 value, int width)
{
	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 19, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, WIDTH, value, 18,  0, 1, NULL,	"Lword Values"	);

	return(0);
}



//*****************************************************************************
static int _rgar_detail(int fd, int supported, u32 value, int width)
{
	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 16, 0, NULL,	"REF Rate Factor"	);
	v	= (value >> 16) & 0xFFFF;
	printf("0x%lX  (Nref %ld)\n", v, v);

	gsc_reg_field_show(width, WIDTH, value, 15,  0, 0, NULL,	"VCO Rate Factor"	);
	v	= value & 0xFFFF;
	printf("0x%lX  (Nvco %ld)\n", v, v);

	return(0);
}



//*****************************************************************************
static int _rdr_detail(int fd, int supported, u32 value, int width)
{
	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 29, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 28, 24, 0, NULL,	"Clock Rate Divisor"	);
	v	= (value >> 24) & 0x1F;
	printf("0x%lX  (Ndiv-clk %ld)\n", v, v);

	gsc_reg_field_show(width, WIDTH, value, 23, 20, 1, NULL,	"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 19,  0, 0, NULL,	"Sync Rate Divisor"		);
	v	= value & 0xFFFFF;
	printf("0x%lX  (Ndiv-sync %ld)\n", v, v);

	return(0);
}



//*****************************************************************************
static int _dfr_detail(int fd, int supported, u32 value, int width)
{
	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 10, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value,  9,  0, 0, NULL,	"Decimation Factor"	);
	v	= value & 0x3FF;
	printf("0x%lX  (Ndec %ld)\n", v, v);

	return(0);
}



//*****************************************************************************
static int _cscr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	holdoff[]	=
	{
		"8 Samples",
		"64 Samples",
		"512 Samples",
		"Infinite"
	};

	static const char*	intigrat[]	=
	{
		"1 Sample",
		"4 Samples",
		"16 Samples",
		"64 Samples"
	};

	static const char*	sync_src[]	=
	{
		"Internal",
		"External Input",
		"Above Sync Level",
		"Below Sync Level",
		"BCR S/W Sync",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	clk_src[]	= { "Internal",		"External"	};
	static const char*	disabled[]	= { "Disabled",		"Enabled"	};
	static const char*	idle[]		= { "Idle",			"Armed"		};
	static const char*	yes[]		= { "Yes",			"No"		};

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 18, 1, NULL,		"Sync Level"				);
	gsc_reg_field_show(width, WIDTH, value, 17, 16, 1, holdoff,		"Sync-On-Level Holdoff"		);
	gsc_reg_field_show(width, WIDTH, value, 15, 14, 1, intigrat,	"Sync-On-Level Integration"	);
	gsc_reg_field_show(width, WIDTH, value, 13, 13, 1, idle,		"Pretrigger Armed"			);
	gsc_reg_field_show(width, WIDTH, value, 12, 12, 1, disabled,	"Pretriggering"				);
	gsc_reg_field_show(width, WIDTH, value, 11, 11, 1, disabled,	"Initiator Outputs"			);
	gsc_reg_field_show(width, WIDTH, value, 10,  8, 1, sync_src,	"Sync Source"				);
	gsc_reg_field_show(width, WIDTH, value,  7,  6, 1, NULL,		"Sync-On-Level Channel"		);
	gsc_reg_field_show(width, WIDTH, value,  5,  5, 1, yes,			"Rate Generator Ready"		);
	gsc_reg_field_show(width, WIDTH, value,  4,  4, 1, clk_src,		"Clock Source"				);
	gsc_reg_field_show(width, WIDTH, value,  3,  3, 1, disabled,	"Channel 3"					);
	gsc_reg_field_show(width, WIDTH, value,  2,  2, 1, disabled,	"Channel 2"					);
	gsc_reg_field_show(width, WIDTH, value,  1,  1, 1, disabled,	"Channel 1"					);
	gsc_reg_field_show(width, WIDTH, value,  0,  0, 1, disabled,	"Channel 0"					);

	return(0);
}



//*****************************************************************************
static int _bsizr_detail(int fd, int supported, u32 value, int width)
{
	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 20, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, WIDTH, value, 19, 16, 0, NULL,	"Burst Size"	);
	v	= value & 0xFFFFF;
	printf("%ld Samples\n", v);

	return(0);
}



//*****************************************************************************
static int _psizr_detail(int fd, int supported, u32 value, int width)
{
	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 18, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, WIDTH, value, 17, 16, 0, NULL,	"Pretrigger Size"	);
	v	= value & 0x3FFFF;
	printf("%ld Scans\n", v);

	return(0);
}



//*****************************************************************************
static int _bcfgr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	in_config[]	=
	{
		"High-Level Inputs",
		"Wideband Inputs",
		"Transformer-Coupled Inputs",
		"RESERVED"
	};

	static const char*	channels[]	= { "4",		"2"		};
	static const char*	impede[]	= { "2 Megohm",	"50 Ohms"	};

	long	v;

	width	+= 10;

	gsc_reg_field_show(width, WIDTH, value, 31, 20, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 19, 19, 1, impede,		"Input Impedence"		);
	gsc_reg_field_show(width, WIDTH, value, 18, 17, 1, in_config,	"Input Configuration"	);
	gsc_reg_field_show(width, WIDTH, value, 16, 16, 1, channels,	"Input Channels"		);
	gsc_reg_field_show(width, WIDTH, value, 15, 12, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, WIDTH, value, 11,  0, 0, NULL,		"Firmware Revision"		);
	v	= value & 0xFFF;
	printf("%03lX\n", v);

	return(0);
}



static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCTLR),		0,	_bctlr_detail,	"Board Control Register"				},
	{ _GSC_REG(PSTSR),		0,	_pstsr_detail,	"Primary Status Register"				},
	{ _GSC_REG(IDBR),		0,	_idbr_detail,	"Input Data Buffer Register"			},
	{ _GSC_REG(IBCR),		0,	_ibcr_detail,	"Input Buffer Control Register"			},
	{ _GSC_REG(BUFSR),		0,	_bufsr_detail,	"Buffer Size Register"					},
	{ _GSC_REG(RGAR),		0,	_rgar_detail,	"Rate Generator Adjust Register"		},
	{ _GSC_REG(RDR),		0,	_rdr_detail,	"Rate Divisors Register"				},
	{ _GSC_REG(DFR),		0,	_dfr_detail,	"Decimation Factor Register"			},
	{ _GSC_REG(CSCR),		0,	_cscr_detail,	"Clocking and Sync Control Register"	},
	{ _GSC_REG(BSIZR),		0,	_bsizr_detail,	"Burst Size Register"					},
	{ _GSC_REG(PSIZR),		0,	_psizr_detail,	"Pretrigger Size Register"				},
	{ _GSC_REG(TUR),		0,	NULL,			"Test Utility Register"					},
	{ _GSC_REG(BCFGR),		0,	_bcfgr_detail,	"Board Configuration Register"			},
	{ _GSC_REG(AVR),		0,	NULL,			"Autocal Values Register"				},
	{ NULL, 0, 0, 0, 0,		NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	hsai4_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* hsai4_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	hsai4_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* hsai4_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	hsai4_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* hsai4_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	hsai4_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* hsai4_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	hsai4_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsai4_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, hsai4_reg_read);
	return(errs);
}



